/*
 *	Network Queueing System (NQS)
 *  This version of NQS is Copyright (C) 1992  John Roman
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
*  PROJECT:     Network Queueing System
*  AUTHOR:      John Roman
*
*  Modification history:
*
*       Version Who     When            Description
*       -------+-------+---------------+-------------------------
*       V01.10  CA                      Initial version.
*       V01.20  JRR     09-Apr-1992	Monsanto modifications.
*	V01.3	JRR	11-Nov-1992	Some fixes.
*/
/*++ qmsg.c - Network Queueing System
 *
 * $Source: /usr2/jrroma/nqs/nqs-3.30/src/RCS/qmsg.c,v $
 *
 * DESCRIPTION:
 *
 *	Write a message to the stdout or stderr file(s) of a batch request.
 *
 *
 *	Author:
 *	-------
 *	Clayton Andreasen, Cray Research, Inc.
 *	July 18, 1986.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision: 1.4 $ $Date: 1992/12/22 15:43:23 $ $State: Exp $)
 * $Log: qmsg.c,v $
 * Revision 1.4  1992/12/22  15:43:23  jrroma
 * Version 3.30
 *
 * Revision 1.3  92/06/17  08:45:38  jrroma
 * Version 3.21
 * 
 * Revision 1.2  92/05/06  10:46:05  jrroma
 *  Version 3.20
 * 
 * Revision 1.1  92/04/09  12:49:08  jrroma
 * Initial revision
 * 
 * Revision 3.2  91/02/11  16:59:05  root
 * Version 2.0 Source
 * 
 * Revision 2.2  87/04/22  15:11:32  hender
 * Sterling version 4/22/87
 * 
 *
 */

#include <fcntl.h>
#include "nqs.h"			/* NQS types and definitions */
#include <errno.h>
#include NETDB				/* Network database files */
#include "nqsdirs.h"			/* NQS files and directories */


#ifndef __CEXTRACT__
#if __STDC__

static void qmsg_usage ( void );

#else /* __STDC__ */

static void qmsg_usage (/* void */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */

/*** main
 *
 *
 *	qmsg [ -eo ] <request-id>
 */
main (argc, argv)
int argc;
char **argv;
{
	int	i;
	char	buffer[4096];		/* message buffer */
	int	errflag;		/* write to stderr flag */
	int	outflag;		/* write to stdout flag */
	int	fd_err;			/* stderr file descriptor */
	int	fd_out;			/* stdout file descriptor */
	long	seqno;			/* Sequence# for req */
	Mid_t	mid;			/* Machine-id of originating machine */
	Mid_t	target_mid;		/*  */
	char	*argument;		/* Ptr to cmd line arg text */
	void	qmsg_usage();		/* Show how to use this command */


	if (chdir (Nqs_root) == -1) {
		fprintf (stderr, "qmgr: Unable to chdir() to the NQS ");
		fprintf (stderr, "root directory.  Errno = %d\n", errno);
		exit (1);
	}
	/*
	 *  Set effective uid to real uid.
	 */
#if UNICOS & NEWPWD
	if (acctid (0,pw->pw_acid[0]) == -1) {
		fprintf (stderr, "Invalid account ID\n");
		exit (1);
	}
#endif
	setuid( getuid() );

	/*
	 *  Interpret flags.
	 */
	while (*++argv != NULL && **argv == '-') {
		argument = *argv;
		while (*++argument) {
			switch (*argument) {
			case 'e':
				errflag = 1;	/* write in stderr */
				break;
			case 'o':		/* write in stdout */
				outflag = 1;
				break;
			default:
				qmsg_usage();
			}
		}
	}
	if (!( errflag | outflag )) errflag = 1;

	/*
	 *  Validate the request-id.
	 */
	if (*argv == NULL) { 		/* If no request-ids was specified */
		fprintf (stderr, "No request-id specified.\n");
		qmsg_usage();
	}
	switch (reqspec (*argv, &seqno, &mid, &target_mid)) {
	case -1:
		fprintf (stderr, "Invalid request-id syntax ");
		fprintf (stderr, "for request-id: %s.\n", *argv);
		exit (1);
	case -2:
		fprintf (stderr, "Unknown machine for");
		fprintf (stderr, "request-id: %s.\n", *argv);
		exit (1);
	case -3:
		fprintf (stderr, "Network mapping database ");
		fprintf (stderr, "inaccessible.  Seek staff ");
		fprintf (stderr, "support.\n");
		exit (1);
	case -4:
		fprintf (stderr, "Network mapping database ");
		fprintf (stderr, "error when parsing ");
		fprintf (stderr, "request-id: %s.\n", *argv);
		fprintf (stderr, "Seek staff support.\n");
		exit (1);
	}

	if (errflag) {
		fd_err = open(namstderr(seqno, mid), O_WRONLY|O_APPEND);
		if (fd_err < 0) {
			errflag = 0;
			if (!outflag) {
				fprintf(stderr, "Error %d opening ", errno);
				fprintf(stderr, "request's stderr file\n");
				exit(1);
			}
		}
	}
	if (outflag) {
		fd_out = open(namstdout(seqno, mid), O_WRONLY|O_APPEND);
		if (fd_out < 0) {
			fprintf(stderr, "Error %d opening ", errno);
			fprintf(stderr, "request's stdout file\n");
			exit(1);
		}
	}
	/*
	 *  Copy the message to stderr and/or stdout.
	 */
	while ((i = read(0, buffer, sizeof(buffer))) > 0) {
		if (errflag) {
			if (write(fd_err, buffer, i) < 0) {
				fprintf(stderr, "Error %d writing to ", errno);
				fprintf(stderr, "request's stderr file\n");
				exit(1);
			}
		}
		if (outflag) {
			if (write(fd_out, buffer, i) < 0) {
				fprintf(stderr, "Error %d writing to ", errno);
				fprintf(stderr, "request's stderr file\n");
				exit(1);
			}
		}
	}
	if (i < 0) {
		fprintf (stderr, "Error %d reading message\n", errno);
		exit(1);
	}
}


/*** qmsg_usage
 *
 *
 *	qmsg_usage():
 *	Show how to use this command.
 */
static void qmsg_usage()
{
	fprintf (stderr, "Usage:  qmsg [ -eo ] request-id\n");
	exit (1);
}
